package cp_r

import (
	"encoding/json"
	"fmt"
	"git.hilo.cn/hilo-common/_const/enum/diamond_e"
	"git.hilo.cn/hilo-common/domain"
	"git.hilo.cn/hilo-common/mycontext"
	"git.hilo.cn/hilo-common/myerr/comerr"
	"git.hilo.cn/hilo-common/rpc"
	"git.hilo.cn/hilo-common/sdk/tencentyun"
	"git.hilo.cn/hilo-common/txop/bag_tx"
	"git.hilo.cn/hilo-common/txop/msg"
	"github.com/gin-gonic/gin"
	"hilo-user/_const/enum/cp_e"
	"hilo-user/cv/cp_cv"
	"hilo-user/domain/model/cp_m"
	"hilo-user/domain/model/diamond_m"
	"hilo-user/domain/model/user_m"
	"hilo-user/myerr/bizerr"
	"hilo-user/req"
	"hilo-user/resp"
	"strconv"
)

// @Tags cp关系
// @Summary 检查用户是否绑定了cp
// @Param code query int true "用户code"
// @Success 200 {object} cp_cv.CheckCpRelationRes
// @Router /v2/cp/relation/check [get]
func CheckUserCpRelation(c *gin.Context) (*mycontext.MyContext, error) {
	myCtx := mycontext.CreateMyContext(c.Keys)
	userCode := c.Query("code")

	_, lang, err := req.GetUserIdLang(c, myCtx)
	if err != nil {
		return myCtx, err
	}

	model := domain.CreateModelContext(myCtx)
	user, err := user_m.GetUserByCode(model, userCode)
	if err != nil {
		return myCtx, err
	}

	cp, err := cp_m.GetCp(model, user.ID)
	if err != nil {
		return myCtx, err
	}
	if cp.Id > 0 {
		return myCtx, msg.GetErrByLanguage(model, 0, lang, comerr.InvalidParameter)
	}

	resp.ResponseOk(c, cp_cv.CheckCpRelationRes{})
	return myCtx, nil
}

// @Tags cp关系
// @Summary 发送cp邀请
// @Param code formData int true "用户code"
// @Success 200
// @Router /v2/cp/relation/invite [post]
func InviteCpRelation(c *gin.Context) (*mycontext.MyContext, error) {
	myCtx := mycontext.CreateMyContext(c.Keys)
	userCode := c.PostForm("code")

	myUserId, lang, err := req.GetUserIdLang(c, myCtx)
	if err != nil {
		return myCtx, err
	}

	model := domain.CreateModelContext(myCtx)
	user, err := user_m.GetUser(model, myUserId)
	if err != nil {
		return myCtx, err
	}
	userInvite, err := user_m.GetUserByCode(model, userCode)
	if err != nil {
		return myCtx, err
	}

	// 自己是否有cp了
	myCp, err := cp_m.GetCp(model, myUserId)
	if err != nil {
		return myCtx, err
	}
	if myCp.Id > 0 {
		return myCtx, msg.GetErrByLanguage(model, 0, lang, comerr.InvalidParameter)
	}
	// 对方是否已经有cp了
	inviCp, err := cp_m.GetCp(model, userInvite.ID)
	if err != nil {
		return myCtx, err
	}
	if inviCp.Id > 0 {
		return myCtx, msg.GetErrByLanguage(model, 0, lang, comerr.InvalidParameter)
	}
	err = model.Transaction(func(model *domain.Model) error {
		// 创建邀请记录
		cpInvId, err := cp_m.CreateCpInvite(model, myUserId, userInvite.ID, cp_e.CpRelationInviteDiamond)
		if err != nil {
			model.Log.Errorf("InviteCpRelation myUserId:%d, err:%v", myUserId, err)
			return err
		}
		// 扣费
		err = diamond_m.ChangeDiamondAccountDetail(model, diamond_e.CpInvite, cpInvId, myUserId, cp_e.CpRelationInviteDiamond)
		if err != nil {
			model.Log.Errorf("InviteCpRelation myUserId:%d, err:%v", myUserId, err)
			return err
		}
		// 发送私信
		data, _ := json.Marshal(cp_m.CpInviteMessage{
			Identifier: "CpInviteMessage",
			Msg:        "Do you want to be CP with me?",
			Status:     uint8(cp_e.CpInvite),
		})
		if err := tencentyun.BatchSendCustomMsg(model, 1, user.ExternalId, []string{userInvite.ExternalId}, string(data), "cp邀请"); err != nil {
			model.Log.Errorf("BatchSendCustomMsg fail:%v", err)
			return err
		}
		return nil
	})
	if err != nil {
		model.Log.Errorf("InviteCpRelation myUserId:%d, err:%v", myUserId, err)
		return myCtx, err
	}
	// socket 推送弹窗
	go rpc.SendCpInviteNotice(userInvite.ID, user.Code, user.Nick, user.Avatar, "Do you want to be CP with me?")

	resp.ResponseOk(c, cp_cv.CheckCpRelationRes{})
	return myCtx, nil
}

// @Tags cp关系
// @Summary 回应cp邀请
// @Param code formData int true "对方的用户code"
// @Param type formData int true "类型1.接受2.拒绝"
// @Success 200
// @Router /v2/cp/relation/invite/reply [post]
func ReplyCpInvite(c *gin.Context) (*mycontext.MyContext, error) {
	myCtx := mycontext.CreateMyContext(c.Keys)
	userCode := c.PostForm("code")
	optType, err := strconv.Atoi(c.PostForm("type"))
	if err != nil || optType > 2 || optType < 1 {
		return myCtx, bizerr.InvalidParameter
	}

	myUserId, lang, err := req.GetUserIdLang(c, myCtx)
	if err != nil {
		return myCtx, err
	}

	model := domain.CreateModelContext(myCtx)
	user, err := user_m.GetUser(model, myUserId)
	if err != nil {
		return myCtx, err
	}
	userSender, err := user_m.GetUserByCode(model, userCode)
	if err != nil {
		return myCtx, err
	}

	cpRecord, err := cp_m.GetCpInvite(model, userSender.ID, user.ID)
	if err != nil {
		model.Log.Errorf("ReplyCpInvite userSender:%d, user:%d, err:%v", userSender.ID, user.ID, err)
		return myCtx, err
	}
	if cpRecord == nil || cpRecord.Id == 0 {
		return myCtx, bizerr.InvalidParameter
	}

	// 自己是否有cp了
	myCp, err := cp_m.GetCp(model, myUserId)
	if err != nil {
		return myCtx, err
	}
	if optType == 1 && myCp.Id > 0 { // 接受的时候
		return myCtx, msg.GetErrByLanguage(model, 0, lang, comerr.InvalidParameter)
	}
	// 对方是否已经有cp了
	senderCp, err := cp_m.GetCp(model, userSender.ID)
	if err != nil {
		return myCtx, err
	}
	if optType == 1 && senderCp.Id > 0 {
		return myCtx, msg.GetErrByLanguage(model, 0, lang, comerr.InvalidParameter)
	}
	err = model.Transaction(func(model *domain.Model) error {
		// 更新邀请状态
		updateStatus := cp_e.CpInviteAccept
		if optType == 2 { // 拒接
			updateStatus = cp_e.CpInviteRefuse
		}
		err = cp_m.UpdateStatusCpInvite(model, cpRecord.Id, updateStatus)
		if err != nil {
			model.Log.Errorf("ReplyCpInvite userSender:%d, user:%d, status:%d, err:%v", userSender.ID, user.ID, updateStatus, err)
			return err
		}

		var msgData []byte
		if optType == 1 { // 接受
			// 写入cp关系表
			err = cp_m.CreateCp(model, userSender.ID, user.ID)
			if err != nil {
				model.Log.Errorf("ReplyCpInvite userSender:%d, user:%d, status:%d, err:%v", userSender.ID, user.ID, updateStatus, err)
				return err
			}
			// 发放告白礼物
			if _, err = bag_tx.SendUserBag(model, userSender.ID, 1, award.GiftId, award.GiftN, 3, "告白礼物"); err != nil {
				model.Log.Errorf("ReplyCpInvite userSender:%d, user:%d, status:%d, err:%v", userSender.ID, user.ID, updateStatus, err)
				return err
			}
			// 私信
			msgData, _ = json.Marshal(cp_m.CpInviteMessage{
				Identifier: "CpInviteMessage",
				Msg:        "We are already CP!",
				Status:     uint8(cp_e.CpInviteAccept),
				Avatar1:    userSender.Avatar,
				Avatar2:    user.Avatar,
			})
		} else { // 拒接
			// 退费
			err = diamond_m.ChangeDiamondAccountDetail(model, diamond_e.CpInviteRefund, cpRecord.Id, cpRecord.UserId, cpRecord.DiamondNum)
			if err != nil {
				model.Log.Errorf("ReplyCpInvite UserId:%d, err:%v", cpRecord.UserId, err)
				return err
			}
			// 私信
			msgData, _ = json.Marshal(cp_m.CpInviteMessage{
				Identifier: "CpInviteMessage",
				Msg:        fmt.Sprintf("%s have declined the CP invitation", user.Nick),
				Status:     uint8(cp_e.CpInviteRefuse),
			})
		}
		if err := tencentyun.BatchSendCustomMsg(model, 1, userSender.ExternalId, []string{user.ExternalId}, string(msgData), "cp邀请"); err != nil {
			model.Log.Errorf("BatchSendCustomMsg fail:%v", err)
			return err
		}
		return nil
	})
	if err != nil {
		model.Log.Errorf("ReplyCpInvite myUserId:%d, err:%v", myUserId, err)
		return myCtx, err
	}

	resp.ResponseOk(c, cp_cv.CheckCpRelationRes{})
	return myCtx, nil
}
